<?php

namespace Drupal\date_format_help\Element;

use Drupal\Core\Render\Attribute\RenderElement;
use Drupal\Core\Render\Element\RenderElementBase;

/**
 * Provides the Date Format Help render Eleement.
 */
#[RenderElement('date_format_help')]
class DateFormatHelp extends RenderElementBase {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return [
      '#pre_render' => [
        [static::class, 'preRenderDateFormHelp'],
      ],
    ];
  }

  /**
   * Returns a list of date formats for the help block.
   *
   * @return array
   *   The grouped list of date formats.
   */
  protected static function dateFormats() {
    return [
      'Day' => [
        'd' => t('Day of the month, 2 digits with leading zeros &mdash; 01 to 31'),
        'D' => t('A textual representation of a day, three letters &mdash; Mon through Sun'),
        'j' => t('Day of the month without leading zeros &mdash; 1 to 31'),
        'l' => t("(lowercase 'L') A full textual representation of the day of the week &mdash; Sunday through Saturday"),
        'N' => t('ISO-8601 numeric representation of the day of the week (added in PHP 5.1.0) &mdash; 1 (for Monday) through 7 (for Sunday)'),
        'S' => t('English ordinal suffix for the day of the month, 2 characters st, nd, rd or th. Works well with j'),
        'w' => t('Numeric representation of the day of the week &mdash; 0 (for Sunday) through 6 (for Saturday)'),
        'z' => t('The day of the year (starting from 0) &mdash; 0 through 365'),
      ],
      'Week' => [
        'W' => t('ISO-8601 week number of year, weeks starting on Monday (added in PHP 4.1.0) Example: 42 (the 42nd week in the year)'),
      ],
      'Month' => [
        'F' => t('A full textual representation of a month, such as January or March &mdash; January through December'),
        'm' => t('Numeric representation of a month, with leading zeros &mdash; 01 through 12'),
        'M' => t('A short textual representation of a month, three letters &mdash; Jan through Dec'),
        'n' => t('Numeric representation of a month, without leading zeros &mdash; 1 through 12'),
        't' => t('Number of days in the given month &mdash; 28 through 31'),
      ],
      'Year'  => [
        'L' => t("Whether it's a leap year 1 if it is a leap year, 0 otherwise."),
        'o' => t('ISO-8601 year number. This has the same value as Y, except that if the ISO week number (W) belongs to the previous or next year, that year is used instead. (added in PHP 5.1.0)  Examples: 1999 or 2003'),
        'Y' => t('A full numeric representation of a year, 4 digits Examples: 1999 or 2003'),
        'y' => t('A two digit representation of a year  Examples: 99 or 03'),
      ],
    ];
  }

  /**
   * Returns a list of date formats for the help block.
   *
   * @return array
   *   The grouped list of date formats.
   */
  protected static function timeFormats() {
    return [
      'Time'  => [
        'a' => t('Lowercase Ante meridiem and Post meridiem &mdash; am or pm'),
        'A' => t('Uppercase Ante meridiem and Post meridiem &mdash; AM or PM'),
        'B' => t('Swatch Internet time &mdash; 000 through 999'),
        'g' => t('12-hour format of an hour without leading zeros &mdash; 1 through 12'),
        'G' => t('24-hour format of an hour without leading zeros &mdash; 0 through 23'),
        'h' => t('12-hour format of an hour with leading zeros &mdash; 01 through 12'),
        'H' => t('24-hour format of an hour with leading zeros &mdash; 00 through 23'),
        'i' => t('Minutes with leading zeros &mdash; 00 to 59'),
        's' => t('Seconds, with leading zeros &mdash; 00 through 59'),
        'u' => t('Microseconds (added in PHP 5.2.2). Note that date() will always generate 000000 since it takes an integer parameter, whereas DateTime::format() does support microseconds.  Example: 654321'),
      ],
      'Timezone' => [
        'e' => t('Timezone identifier (added in PHP 5.1.0)  Examples: UTC, GMT, Atlantic/Azores'),
        'I' => t('(capital i) Whether or not the date is in daylight saving time  1 if Daylight Saving Time, 0 otherwise.'),
        'O' => t('Difference to Greenwich time (GMT) in hours Example: +0200'),
        'P' => t('Difference to Greenwich time (GMT) with colon between hours and minutes (added in PHP 5.1.3)  Example: +02:00'),
        'T' => t('Timezone abbreviation Examples: EST, MDT ...'),
        'Z' => t('Timezone offset in seconds. The offset for timezones west of UTC is always negative, and for those east of UTC is always positive. &mdash; -43200 through 50400'),
      ],
      'Full Date/Time' => [
        'c' => t('ISO 8601 date (added in PHP 5)  2004-02-12T15:19:21+00:00'),
        'r' => t('» RFC 2822 formatted date Example: Thu, 21 Dec 2000 16:01:07 +0200'),
        'U' => t('Seconds since the Unix Epoch (January 1 1970 00:00:00 GMT)  See also time()'),
      ],
    ];
  }

  /**
   * Generate the render array for the help block.
   *
   * Convert the date/time formats to set of tables. The $formats
   * variable should be an array of sections, with the section title as
   * the key and the value is an array of available format options.
   *
   * The array of format options is keyed with the format option, and the
   * value is the help text.
   *
   * Example:
   *
   * [
   *   'Year' => [
   *     'y' => t('A two digit representation of a year  Examples: 99 or 03'),
   *   ),
   * )
   *
   * @param array $element
   *   An associative array containing the properties and children of the
   *   container.
   *
   * @return array
   *   The modified element.
   */
  public static function preRenderDateFormHelp(array $element): array {

    $format_descriptions = [
      static::class::dateFormats(),
      static::class::timeFormats(),
    ];

    foreach ($format_descriptions as $formats) {
      $tables = [
        '#type' => 'container',
        '#prefix' => '<div class="time-formats-help">',
        '#suffix' => '</div>',
      ];

      foreach ($formats as $header => $formatters) {
        $formatter_table = [];
        foreach ($formatters as $key => $value) {
          $formatter_table[] = [
            $key,
            $value,
            date($key),
          ];
        }
        $prefix = '<div class="date-time-formatter-help-block">';
        $prefix .= '<h3>' . $header . '</h3>';
        $tables[] = [
          '#prefix' => $prefix,
          '#suffix' => '</div>',
          '#theme' => 'table',
          '#rows' => $formatter_table,
        ];
      }
      $element[] = $tables;
    }
    $element['#attached']['library'][] = 'date_format_help/date_format_add';

    return $element;
  }

}
